@@ -9,14 +9,14 @@ use Math::Prime::Util qw/is_prob_prime is_strong_pseudoprime
                          prime_get_config
                         /;
 
-BEGIN {
-  $Math::Prime::Util::PrimalityProving::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::PrimalityProving::VERSION = '0.44_004';
+if (!defined $Math::BigInt::VERSION) {
+  eval { require Math::BigInt;   Math::BigInt->import(try=>'GMP,Pari'); 1; }
+  or do { croak "Cannot load Math::BigInt"; };
 }
 
 BEGIN {
-  do { require Math::BigInt;  Math::BigInt->import(try=>"GMP,Pari"); }
-    unless defined $Math::BigInt::VERSION;
+  $Math::Prime::Util::PrimalityProving::AUTHORITY = 'cpan:DANAJ';
+  $Math::Prime::Util::PrimalityProving::VERSION = '0.30';
 }
 
 my $_smallval = Math::BigInt->new("18446744073709551615");
@@ -57,14 +57,6 @@ sub _small_cert {
                     "";
 }
 
-# For stripping off the header on certificates so they can be combined.
-sub _strip_proof_header {
-  my $proof = shift;
-  $proof =~ s/^\[MPU - Primality Certificate\]\nVersion \S+\n+Proof for:\nN (\d+)\n+//ms;
-  return $proof;
-}
-
-
 sub primality_proof_lucas {
   my ($n) = shift;
   my @composite = (0, '');
@@ -74,6 +66,11 @@ sub primality_proof_lucas {
   return (2, _small_cert($n)) if $n < 4;
   return @composite if is_strong_pseudoprime($n,2,15,325) == 0;
 
+  if (!defined $Math::BigInt::VERSION) {
+    eval { require Math::BigInt;   Math::BigInt->import(try=>'GMP,Pari'); 1; }
+    or do { croak "Cannot load Math::BigInt"; };
+  }
+
   my $nm1 = $n-1;
   my @factors = factor($nm1);
   { # remove duplicate factors and make a sorted array of bigints
@@ -104,7 +101,7 @@ sub primality_proof_lucas {
         carp "could not prove primality of $n.\n";
         return (1, '');
       }
-      push @fac_proofs, _strip_proof_header($fproof) if $f > $_smallval;
+      push @fac_proofs, Math::Prime::Util::_strip_proof_header($fproof) if $f > $_smallval;
     }
     $cert .= "A $a\n";
     foreach my $proof (@fac_proofs) {
@@ -125,25 +122,17 @@ sub primality_proof_bls75 {
   return @composite if ($n & 1) == 0;
   return @composite if is_strong_pseudoprime($n,2,15,325) == 0;
 
-  require Math::Prime::Util::PP;
   $n = Math::BigInt->new("$n") unless ref($n) eq 'Math::BigInt';
-  my $nm1 = $n->copy->bdec;
-  my $ONE = $nm1->copy->bone;
-  my $TWO = $ONE->copy->binc;
-  my $A = $ONE->copy;         # factored part
+  my $nm1 = $n-1;
+  my $A = $nm1->copy->bone;   # factored part
   my $B = $nm1->copy;         # unfactored part
-  my @factors = ($TWO);
+  my @factors = (2);
   croak "BLS75 error: n-1 not even" unless $nm1->is_even();
+  my $trial_B = 20000;
   {
-    while ($B->is_even) { $B->bdiv($TWO); $A->bmul($TWO); }
-    my @tf;
-    if ($B <= ''.~0 && prime_get_config->{'xs'}) {
-      @tf = Math::Prime::Util::trial_factor($B, 20000);
-      pop @tf if $tf[-1] > 20000;
-    } else {
-      @tf = Math::Prime::Util::PP::trial_factor($B, 500);
-      pop @tf if $tf[-1] > 500;
-    }
+    while ($B->is_even) { $B /= 2; $A *= 2; }
+    my @tf = Math::Prime::Util::PP::trial_factor($B, $trial_B);
+    pop @tf if $tf[-1] > $trial_B;
     foreach my $f (@tf) {
       next if $f == $factors[-1];
       push @factors, $f;
@@ -152,7 +141,7 @@ sub primality_proof_bls75 {
   }
   my @nstack;
   # nstack should only hold composites
-  if ($B->is_one) {
+  if ($B == 1) {
     # Completely factored.  Nothing.
   } elsif (is_prob_prime($B)) {
     push @factors, $B;
@@ -161,17 +150,13 @@ sub primality_proof_bls75 {
     push @nstack, $B;
   }
   while (@nstack) {
-    my ($s,$r) = $B->copy->bdiv($A->copy->bmul($TWO));
-    my $fpart = ($A+$ONE) * ($TWO*$A*$A + ($r-$ONE) * $A + $ONE);
+    my ($s,$r) = $B->copy->bdiv($A->copy->bmul(2));
+    my $fpart = ($A+1) * (2*$A*$A + ($r-1) * $A + 1);
     last if $n < $fpart;
 
     my $m = pop @nstack;
     # Don't use bignum if it has gotten small enough.
-    if ($] < 5.008) {
-      $m = int($m->bstr) if ref($m) eq 'Math::BigInt' && $m <= 562949953421312;
-    } else {
-      $m = int($m->bstr) if ref($m) eq 'Math::BigInt' && $m <= ''.~0;
-    }
+    $m = int($m->bstr) if ref($m) eq 'Math::BigInt' && $m <= ''.~0;
     # Try to find factors of m, using the default set of factor subs.
     my @ftry;
     foreach my $sub (@_fsublist) {
@@ -182,33 +167,33 @@ sub primality_proof_bls75 {
     next unless scalar @ftry > 1;
     # Process each factor
     foreach my $f (@ftry) {
-      croak "Invalid factoring: B=$B m=$m f=$f" if $f == 1 || $f == $m || !$B->copy->bmod($f)->is_zero;
+      croak "Invalid factoring: B=$B m=$m f=$f" if $f == 1 || $f == $m || ($B%$f) != 0;
       if (is_prob_prime($f)) {
         push @factors, $f;
-        do { $B /= $f;  $A *= $f; } while $B->copy->bmod($f)->is_zero;
+        do { $B /= $f;  $A *= $f; } while (($B % $f) == 0);
       } else {
         push @nstack, $f;
       }
     }
   }
+  # Just in case:
+  foreach my $f (@factors) {
+    while (($B % $f) == 0) {
+      $B /= $f;  $A *= $f;
+    }
+  }
   { # remove duplicate factors and make a sorted array of bigints
     my %uf = map { $_ => 1 } @factors;
     @factors = sort {$a<=>$b} map { Math::BigInt->new("$_") } keys %uf;
   }
-  # Just in case:
-  foreach my $f (@factors) {
-    while ($B->copy->bmod($f)->is_zero) {
-      $B /= $f;  $A *= $f;
-    }
-  }
   # Did we factor enough?
-  my ($s,$r) = $B->copy->bdiv($A->copy->bmul($TWO));
-  my $fpart = ($A+$ONE) * ($TWO*$A*$A + ($r-$ONE) * $A + $ONE);
+  my ($s,$r) = $B->copy->bdiv($A->copy->bmul(2));
+  my $fpart = ($A+1) * (2*$A*$A + ($r-1) * $A + 1);
   return (1,'') if $n >= $fpart;
   # Check we didn't mess up
   croak "BLS75 error: $A * $B != $nm1" unless $A*$B == $nm1;
   croak "BLS75 error: $A not even" unless $A->is_even();
-  croak "BLS75 error: A and B not coprime" unless Math::BigInt::bgcd($A, $B)->is_one;
+  croak "BLS75 error: A and B not coprime" unless Math::BigInt::bgcd($A, $B)==1;
 
   my $rtest = $r*$r - 8*$s;
   my $rtestroot = $rtest->copy->bsqrt;
@@ -226,11 +211,10 @@ sub primality_proof_bls75 {
     } else {
       $cert .= "Q[$qnum] $f\n";
     }
-    my $nm1_div_f = $nm1 / $f;
     foreach my $a (2 .. 10000) {
       my $ap = Math::BigInt->new($a);
-      next unless $ap->copy->bmodpow($nm1, $n)->is_one;
-      next unless Math::BigInt::bgcd($ap->copy->bmodpow($nm1_div_f, $n)->bdec, $n)->is_one;
+      next unless $ap->copy->bmodpow($nm1, $n) == 1;
+      next unless Math::BigInt::bgcd($ap->copy->bmodpow($nm1/$f, $n)->bsub(1), $n) == 1;
       $atext .= "A[$qnum] $a\n" unless $a == 2;
       $success = 1;
       last;
@@ -242,7 +226,7 @@ sub primality_proof_bls75 {
       carp "could not prove primality of $n.\n";
       return (1, '');
     }
-    push @fac_proofs, _strip_proof_header($fproof) if $f > $_smallval;
+    push @fac_proofs, Math::Prime::Util::_strip_proof_header($fproof) if $f > $_smallval;
   }
   $cert .= $atext;
   $cert .= "----\n";
@@ -393,12 +377,12 @@ sub convert_array_cert_to_string {
 ###############################################################################
 
 sub _primality_error ($) {  ## no critic qw(ProhibitSubroutinePrototypes)
-  print "primality fail: $_[0]\n" if prime_get_config->{'verbose'};
+  print "primality fail: $_[0]" if prime_get_config->{'verbose'};
   return;  # error in certificate
 }
 
 sub _pfail ($) {            ## no critic qw(ProhibitSubroutinePrototypes)
-  print "primality fail: $_[0]\n" if prime_get_config->{'verbose'};
+  print "primality fail: $_[0]" if prime_get_config->{'verbose'};
   return;  # Failed a condition
 }
 
@@ -576,8 +560,8 @@ sub _verify_ecpp {
     unless Math::BigInt::bgcd(4*$a*$a*$a+27*$b*$b,$n) == 1;
   return _pfail "ECPP: $n failed Y^2 = X^3 + A*X + B mod N"
     unless ($y*$y) % $n == ($x*$x*$x + $a*$x + $b) % $n;
-  return _pfail "ECPP: $n failed M >= N - 2*sqrt(N) + 1" unless $m >= $n + 1 - $n->copy->bmul(4)->bsqrt();
-  return _pfail "ECPP: $n failed M <= N + 2*sqrt(N) + 1" unless $m <= $n + 1 + $n->copy->bmul(4)->bsqrt();
+  return _pfail "ECPP: $n failed M >= N - 2*sqrt(N) + 1" unless $m >= $n - 2*$n->copy->bsqrt() + 1;
+  return _pfail "ECPP: $n failed M <= N + 2*sqrt(N) + 1" unless $m <= $n + 2*$n->copy->bsqrt() + 1;
   return _pfail "ECPP: $n failed Q > (N^(1/4)+1)^2" unless $q > $n->copy->broot(4)->badd(1)->bpow(2);
   return _pfail "ECPP: $n failed Q < N" unless $q < $n;
   return _pfail "ECPP: $n failed M != Q" unless $m != $q;
@@ -731,9 +715,9 @@ sub _verify_bls5 {
     my $a = $A[$i];
     my $q = $Q[$i];
     return _pfail "BLS5: $n failed A[i]^(N-1) mod N = 1"
-      unless $a->copy->bmodpow($nm1, $n)->is_one;
+      unless $a->copy->bmodpow($nm1, $n) == 1;
     return _pfail "BLS5: $n failed gcd(A[i]^((N-1)/Q[i])-1, N) = 1"
-      unless Math::BigInt::bgcd($a->copy->bmodpow($nm1/$q, $n)->bdec, $n)->is_one;
+      unless Math::BigInt::bgcd($a->copy->bmodpow($nm1/$q, $n)-1, $n) == 1;
   }
   ($n, @Q);
 }
@@ -803,10 +787,7 @@ sub verify_cert {
     if ( ($cert_type eq 'PRIMO' && $line =~ /^\[Candidate\]/) || ($cert_type eq 'MPU' && $line =~ /^Proof for:/) ) {
       return _primality_error "Certificate with multiple N values" if defined $N;
       ($N) = _read_vars($lines, 'Proof for', qw/N/);
-      if (!is_prob_prime($N)) {
-        _pfail "N '$N' does not look prime.";
-        return 0;
-      }
+      return 0 unless is_prob_prime($N);
       next;
     }
     if ($line =~ /^Base (\d+)/) {
@@ -868,7 +849,7 @@ Math::Prime::Util::PrimalityProving - Primality proofs and certificates
 
 =head1 VERSION
 
-Version 0.44_004
+Version 0.30
 
 
 =head1 SYNOPSIS
